#[macro_use]
extern crate diesel;
extern crate dotenv;

pub mod model;
pub mod schema;

use diesel::prelude::*;
use diesel::result::{DatabaseErrorKind, Error};

use dotenv::dotenv;
use std::env;

use self::model::*;

use time::OffsetDateTime;

use crypto::digest::Digest;
use crypto::sha2::Sha256;
use rand::{thread_rng, Rng};

pub enum UserCreationStatus {
    Success,
    UserExists,
    QueryError(Error),
    DatabaseError(DatabaseErrorKind),
}

pub enum UserSelectionStatus<T> {
    Success(T),
    UserDoesntExist,
    QueryError(Error),
    DatabaseError(DatabaseErrorKind)
}

pub fn create_connection() -> MysqlConnection {
    dotenv().ok();

    let database_url = env::var("DATABASE_URL").expect("Database URL Not Defined");
    MysqlConnection::establish(&database_url).unwrap()
}

pub fn get_id_from_email(user_email: String, connection: &mut MysqlConnection) -> UserSelectionStatus<i64> {
    use schema::users::dsl::*;
    
    let users_matching_email = match users
        .filter(email.eq(user_email))
        .limit(1)
        .select(User::as_select())
        .load(connection) {
        Ok(v) => v,
        Err(_e) => vec![]
    };

    let user = match users_matching_email.get(0) {
        Some(v) => v,
        None => return UserSelectionStatus::UserDoesntExist 
    };
    
    UserSelectionStatus::Success(user.id)
}

pub fn check_password(
    user_id: i64,
    password: String,
    connection: &mut MysqlConnection,
) -> UserSelectionStatus<bool> {

    use self::schema::users::dsl::*;

    let users_matching = match users
        .filter(id.eq(user_id))
        .limit(1)
        .select(User::as_select())
        .load(connection) {
        Ok(v) => v,
        Err(_e) => vec![]
    };

    let user = match users_matching.get(0) {
        Some(v) => v,
        None => return UserSelectionStatus::UserDoesntExist
    };

    let mut hasher = Sha256::new();
    hasher.input_str(&(password + &user.salt));

    let hashed = hasher.result_str();

    UserSelectionStatus::Success(user.pass == hashed)
}

pub fn add_user(
    username: String,
    email: String,
    password: String,
    connection: &mut MysqlConnection,
) -> UserCreationStatus {
    use schema::users::dsl::users;

    let joined = OffsetDateTime::now_utc().date();

    let salt: String = {
        const CHARSET: &[u8] = b"abcdef0123456789";

        let mut rng = thread_rng();
        let salt: String = (0..64)
            .map(|_| {
                let idx = rng.gen_range(0..CHARSET.len());
                CHARSET[idx] as char
            })
            .collect();
        salt
    };

    let mut hasher = Sha256::new();
    hasher.input_str(&(password + &salt));

    let pass = hasher.result_str();

    let new_user = model::NewUser {
        username,
        email,
        salt,
        pass,
        joined,
    };

    match diesel::insert_into(users)
        .values(&new_user)
        .execute(connection)
    {
        Ok(_) => UserCreationStatus::Success,
        Err(e) => match e {
            Error::DatabaseError(database_error_kind, _) => match database_error_kind {
                DatabaseErrorKind::UniqueViolation => UserCreationStatus::UserExists,
                _ => UserCreationStatus::DatabaseError(database_error_kind),
            },
            _ => UserCreationStatus::QueryError(e),
        },
    }
}
